# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from knack.util import CLIError
from azure.cli.core.azclierror import ValidationError
from azure.core.exceptions import HttpResponseError
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
import time
LOCATION = "eastus2"
#LOCATION = "eastus2euap"
ADLOCATION = "northeurope"
#ADLOCATION = "eastus2euap"

# No tidy up of tests required. The resource group is automatically removed


class AzureNetAppFilesAccountServiceScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_create_delete_account(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'tags': 'Tag1=Value1 Tag2=Value2'
        })

        # create and check
        # note : active directory checks are performed in their own subgroup test
        self.cmd(
            "az netappfiles account create --resource-group {rg} --account-name '{acc_name}' --location {loc} "
            "--tags {tags}", checks=[
                self.check('name', '{acc_name}'),
                self.check('tags.Tag1', 'Value1'),
                self.check('tags.Tag2', 'Value2')
            ])

        self.cmd("netappfiles account list --resource-group {rg}", checks=[
            self.check('length(@)', 1)
        ])

        # delete and recheck
        self.cmd("az netappfiles account delete --resource-group {rg} --account-name '{acc_name}' --yes")
        self.cmd("netappfiles account list --resource-group {rg}", checks=[
            self.check('length(@)', 0)
        ])

        # and again with short forms and also unquoted
        self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --tags {tags}", checks=[
            self.check('name', '{acc_name}')
        ])
        self.cmd("netappfiles account list --resource-group {rg}", checks=[
            self.check('length(@)', 1)
        ])

        self.cmd("az netappfiles account delete --resource-group {rg} -a {acc_name} --yes")
        self.cmd("netappfiles account list --resource-group {rg}", checks=[
            self.check('length(@)', 0)
        ])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_list_accounts(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc1_name': self.create_random_name(prefix='cli-acc-', length=24),
            'acc2_name': self.create_random_name(prefix='cli-acc-', length=24)
        })
        self.cmd("az netappfiles account create -g {rg} -a {acc1_name} -l {loc} --tags Tag1=Value1")
        self.cmd("az netappfiles account create -g {rg} -a {acc2_name} -l {loc} --tags Tag1=Value1")

        self.cmd("netappfiles account list -g {rg}", checks=[
            self.check('length(@)', 2)
        ])

        self.cmd("az netappfiles account delete -g {rg} -a {acc1_name} --yes")
        self.cmd("az netappfiles account delete -g {rg} -a {acc2_name} --yes")

        self.cmd("netappfiles account list -g {rg}", checks=[
            self.check('length(@)', 0)
        ])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_get_account_by_name(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24)
        })
        self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc}")
        account = self.cmd("az netappfiles account show --resource-group {rg} -a {acc_name}", checks=[
            self.check('name', '{acc_name}')
        ]).get_output_in_json()
        # test get account from id
        self.cmd(("az netappfiles account show --ids %s" % account['id']), checks=[self.check('name', '{acc_name}')])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_update_account(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'tags': 'Tag1=Value1 Tag2=Value2'
        })

        # create, update and check
        self.cmd(
            "az netappfiles account create -g {rg} -a '{acc_name}' -l {loc}", checks=[self.check('name', '{acc_name}')])
        self.cmd("az netappfiles account update -g {rg} -a {acc_name} --tags {tags}", checks=[
            self.check('name', '{acc_name}'),
            self.check('tags.Tag1', 'Value1'),
            self.check('tags.Tag2', 'Value2')
        ])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_active_directory(self):
        self.kwargs.update({
            'loc': ADLOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'ad_name': 'cli-ad-name',
            'kdc_ip': '172.016.254.1',
            'ldap': True,
            'ldap_users': True,
            'ad_user': 'ad_user'
        })
        # create account
        self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --tags Tag1=Value1", checks=[
            self.check('name', '{acc_name}')
        ])

        account = self.cmd("az netappfiles account show --resource-group {rg} -a {acc_name}", checks=[
            self.check('name', '{acc_name}')
        ]).get_output_in_json()

        if self.is_live or self.in_recording:
            time.sleep(60)
        # add an active directory
        self.cmd(
            "netappfiles account ad add -g {rg} -n {acc_name} --username {ad_user} --password {ad_user} "
            "--smb-server-name SMBSERVER --dns '1.2.3.4' --domain {loc} --ad-name {ad_name} --kdc-ip {kdc_ip} "
            "--ldap-signing {ldap} --allow-local-ldap-users {ldap_users}")
        # self.cmd(
        #     "netappfiles account ad add -g {rg} -n {acc_name} --username {ad_user} --password {ad_user} "
        #     "--smb-server-name SMBSERVER --dns '1.2.3.4' --domain {loc} --ad-name {ad_name} --kdc-ip {kdc_ip} "
        #     "--ldap-signing {ldap} --allow-local-ldap-users {ldap_users}", checks=[
        #         self.check('name', '{acc_name}'),
        #         self.check('activeDirectories[0].username', '{ad_user}'),
        #         self.check('activeDirectories[0].status', 'Created'),
        #         self.check('activeDirectories[0].adName', '{ad_name}'),
        #         self.check('activeDirectories[0].aesEncryption', False),
        #         self.check('activeDirectories[0].ldapSigning', '{ldap}'),
        #         self.check('activeDirectories[0].allowLocalNfsUsersWithLdap', '{ldap_users}')
        #     ])

        # list active directory
        active_directories = self.cmd("netappfiles account ad list -g {rg} -n {acc_name}", checks=[
            self.check('[0].username', '{ad_user}'),
            self.check('length(@)', 1)
        ]).get_output_in_json()

        self.kwargs.update({
            'ad_id': active_directories[0]['activeDirectoryId']
        })

        # update active directory
        self.cmd("az netappfiles account ad update -g {rg} -n {acc_name} --active-directory-id {ad_id} "
                 "--password {ad_user} --username {ad_user} "
                 "--smb-server-name SMBSERVER --dns '1.2.3.5' --domain {loc} --ad-name {ad_name} --kdc-ip {kdc_ip} "
                 "--ldap-signing {ldap} --allow-local-ldap-users {ldap_users}",
                 checks=[
                     self.check('name', '{acc_name}'),
                     self.check('activeDirectories[0].username', '{ad_user}'),
                     self.check('activeDirectories[0].status', 'Created'),
                     self.check('activeDirectories[0].adName', '{ad_name}'),
                     self.check('activeDirectories[0].aesEncryption', False),
                     self.check('activeDirectories[0].ldapSigning', '{ldap}'),
                     #self.check('activeDirectories[0].allowLocalNFSUsersWithLdap', '{ldap_users}')
                 ])

        # remove active directory using the previously obtained details
        self.cmd("netappfiles account ad remove -g {rg} -n {acc_name} --active-directory %s --yes" %
                 active_directories[0]['activeDirectoryId'])

        self.cmd("netappfiles account show -g {rg} -n {acc_name}", checks=[
            self.check('name', '{acc_name}'),
            self.check('activeDirectories', None)
        ])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_account_encryption(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'acc2_name': self.create_random_name(prefix='cli-acc-', length=24),
            'encryption': "Microsoft.NetApp"
        })
        # create account with encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --key-source {encryption}", checks=[
            self.check('name', '{acc_name}'),
            self.check('encryption.keySource', '{encryption}')
        ])

        # create account without encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc2_name} -l {loc}", checks=[
            self.check('name', '{acc2_name}')
        ])

        # update account with encryption value
        self.cmd("az netappfiles account update -g {rg} -a {acc2_name} --key-source {encryption}", checks=[
            self.check('name', '{acc2_name}'),
            self.check('encryption.keySource', '{encryption}')
        ])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_account_cmk_encryption(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'acc2_name': self.create_random_name(prefix='cli-acc-', length=24),
            'keySource': "Microsoft.KeyVault",
            'keyVaultUri': "myUri",
            'keyName': "myKeyName",
            'keyVaultResourceId': "myKeyVaultResourceId",
            'userAssignedIdentity': "myIdentity",
            'identityType': "UserAssigned"
        })

        with self.assertRaises(HttpResponseError):
            # create account with encryption value                                                                                           user-assigned-identity
            self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --key-source {keySource} --identity-type {identityType} --user-assigned-identity {userAssignedIdentity} --key-vault-uri {keyVaultUri} --key-name {keyName} --keyvault-resource-id {keyVaultResourceId} --user-assigned-identity {userAssignedIdentity}", checks=[
                self.check('name', '{acc_name}'),
                self.check('encryption.keySource', '{keySource}'),
                self.check('identity.type', '{identityType}')
            ])

        # create account without encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc2_name} -l {loc}", checks=[
            self.check('name', '{acc2_name}')
        ])

        # update account with encryption value
        with self.assertRaises(HttpResponseError):
            self.cmd("az netappfiles account update -g {rg} -a {acc2_name} --key-source {keySource} --identity-type {identityType} --user-assigned-identity {userAssignedIdentity}", checks=[
                self.check('name', '{acc2_name}'),
                self.check('encryption.keySource', '{keySource}')
            ])

    #@unittest.skip('(servicedeployment) api has not been deployed cannot test until finalized')
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_account_renew_credentials_fails(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'acc2_name': self.create_random_name(prefix='cli-acc-', length=24),
            'keySource': "Microsoft.KeyVault",
            'keyVaultUri': "myUri",
            'keyName': "myKeyName",
            'keyVaultResourceId': "myKeyVaultResourceId",
            'userAssignedIdentity': "myIdentity"
        })

        with self.assertRaises(HttpResponseError):
            # create account with encryption value
            self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --key-source {keySource} --key-vault-uri {keyVaultUri} --key-name {keyName} --keyvault-resource-id {keyVaultResourceId} --user-assigned-identity {userAssignedIdentity}", checks=[
                self.check('name', '{acc_name}'),
                self.check('encryption.keySource', '{keySource}')
            ])

        # create account without encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc2_name} -l {loc}", checks=[
            self.check('name', '{acc2_name}')
        ])

        with self.assertRaises(HttpResponseError) as cm:
            # create account with encryption value
            self.cmd("az netappfiles account renew-credentials -g {rg} -a {acc2_name} ", checks=[
                self.check('name', '{acc2_name}'),
            ])
        self.assertIn('MsiInvalidForRenewal', str(
            cm.exception))
        # with self.assertRaises(CLIError):
        #     # create account with encryption value
        #     self.cmd("az rest --method POST --uri /subscriptions/69a75bda-882e-44d5-8431-63421204132a/resourcegroups/{rg}/providers/Microsoft.NetApp/netappAccounts/{acc_name}/renewCredentials?api-version=2022-05-01  ", checks=[
        #         self.check('name', '{acc_name}'),
        #     ])

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'}, location='eastus2euap')
    def test_create_account_with_no_location(self):
        self.kwargs.update({
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24)            
        })
        location = "eastus2euap"
        self.cmd("az netappfiles account create -g {rg} -a {acc_name}")
        self.cmd("az netappfiles account show --resource-group {rg} -a {acc_name}", checks=[
            self.check('location', location)
        ])
    
    @unittest.skip('(servicedeployment) api has not been deployed cannot test until finalized')
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_account_transitionCMK_fails(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'acc2_name': self.create_random_name(prefix='cli-acc-', length=24),
            'keySource': "Microsoft.KeyVault",
            'keyVaultUri': "https://my-key-vault.managedhsm.azure.net",
            'keyName': "myKeyName",
            'keyVaultResourceId': "/subscriptions/69a75bda-882e-44d5-8431-63421204132a/resourceGroups/myRG/providers/Microsoft.KeyVault/managedHSMs/my-hsm",
            'userAssignedIdentity': "myIdentity",
            'privateEndpointId': '/subscriptions/69a75bda-882e-44d5-8431-63421204132a/resourceGroups/ab_sdk_test_rg/providers/Microsoft.Network/privateEndpoints/akvPrivateEndpoint',
            'virtualNetworkId': '/subscriptions/69a75bda-882e-44d5-8431-63421204132a/resourceGroups/ab_sdk_test_rg/providers/Microsoft.Network/virtualNetworks/ab_sdk_test_vnet'            
        })

        with self.assertRaises(HttpResponseError):
            # create account with encryption value
            self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --key-source {keySource} --key-vault-uri {keyVaultUri} --key-name {keyName} --keyvault-resource-id {keyVaultResourceId} --user-assigned-identity {userAssignedIdentity}", checks=[
                self.check('name', '{acc_name}'),
                self.check('encryption.keySource', '{keySource}')
            ])

        # create account without encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc2_name} -l {loc}", checks=[
            self.check('name', '{acc2_name}')
        ])
        
        # create account with encryption value
        self.cmd("az netappfiles account get-key-vault-status -g {rg} -a {acc2_name} ")
        
        with self.assertRaises(HttpResponseError) as cm:
            # create account with encryption value
            self.cmd("az netappfiles account transitiontocmk -g {rg} -a {acc2_name} --private-endpoint-id {privateEndpointId} --virtual-network-id {virtualNetworkId}  --yes")
        self.assertIn('AccountEncryptionInvalidForTransitionEncryption', str(
            cm.exception)) 
        
    @unittest.skip('(servicedeployment) api has not been deployed cannot test until finalized')
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_account_', additional_tags={'owner': 'cli_test'})
    def test_account_changekeyvault_fails(self):
        self.kwargs.update({
            'loc': LOCATION,
            'acc_name': self.create_random_name(prefix='cli-acc-', length=24),
            'acc2_name': self.create_random_name(prefix='cli-acc-', length=24),
            'pool_name': self.create_random_name(prefix='cli-acc-', length=24),
            'keyVaultName': self.create_random_name(prefix='cli-acc-', length=24),
            'vnetName': self.create_random_name(prefix='cli-acc-', length=24),
            'subnetName': self.create_random_name(prefix='cli-acc-', length=24),
            'privateEndpointName': self.create_random_name(prefix='cli-acc-', length=24),
            'connectionName': self.create_random_name(prefix='cli-acc-', length=24),
            'keySource': "Microsoft.KeyVault",
            'keyName': "myKeyName",            
            'userAssignedIdentity': "myIdentity"
        })

        keyVault = self.cmd("az keyvault create --resource-group {rg} --name {keyVaultName}  --location {loc} --enable-purge-protection --retention-days 7 --enable-rbac-authorization").get_output_in_json()        
        self.kwargs.update({
            'keyVaultResourceId': keyVault.get_output_in_json()['id'],
            'keyVaultUri': keyVault.get_output_in_json()['properties']['vaultUri']            
        })
        # vnet = self.cmd("az network vnet create -g {rg} -n {vnetName} -l {loc} --address-prefixes 10.0.0.0/16").get_output_in_json()
        # self.cmd("az network vnet subnet create -g {rg} --vnet-name {vnetName} -n {subnetName} --address-prefixes '10.0.2.0/24'")
        # endpoint= self.cmd("az network private-endpoint create -g {rg} --vnet-name {vnetName} --subnet {subnetName} --name {privateEndpointName} --group-ids vault --connection-name {connectionName} --location {loc} --private-connection-resource-id {keyVaultResourceId}").get_output_in_json()
        # self.cmd("az network vnet subnet update -g {rg} --vnet-name {vnetName} --name {subnetName} --disable-private-endpoint-network-policies true")
        
        # self.kwargs.update({        
        #     'privateEndpointId': endpoint.get_output_in_json()['id'],
        #     'virtualNetworkId': '/subscriptions/69a75bda-882e-44d5-8431-63421204132a/resourceGroups/ab_sdk_test_rg/providers/Microsoft.Network/virtualNetworks/ab_sdk_test_vnet'
        # })
        
        
        self.cmd("az keyvault key create --vault-name {keyVaultName} -n {keyName} --kty RSA --protection software")
        self.cmd("az keyvault update -g ab_sdk_test_rg -n {keyVaultName} --default-action deny")
        
        # create account with encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc_name} -l {loc} --key-source {keySource} --key-vault-uri {keyVaultUri} --key-name {keyName} --keyvault-resource-id {keyVaultResourceId} --identity-type SystemAssigned", checks=[
            self.check('name', '{acc_name}'),
            self.check('encryption.keySource', '{keySource}')
        ])
        principalId = self.cmd("az netappfiles account show -g {rg} -a {acc_name} ").get_output_in_json()['identity'].principalId
        self.kwargs.update({
            'principalId': principalId
         })
        self.cmd("az keyvault set-policy -n {keyVaultName} --key-permissions get encrypt Decrypt  --object-id {principalId}")
        # create account without encryption value
        self.cmd("az netappfiles account create -g {rg} -a {acc2_name} -l {loc}", checks=[
            self.check('name', '{acc2_name}')
        ])
        
        self.cmd("az netappfiles pool create -g {rg} -a {acc_name} -p {pool_name} -l {loc} --service-level 'Premium' --size 4").get_output_in_json()

        # Get keyvault status
        keyVaultStatus = self.cmd("az netappfiles account get-key-vault-status -g {rg} -a {acc2_name} ").get_output_in_json()
        self.kwargs.update({
            'principalId': keyVaultStatus.keyVaultPrivateEndpoints[0].privateEndpointId,
            'virtualNetworkId': keyVaultStatus.keyVaultPrivateEndpoints[0].virtualNetworkId
         })        
                
        with self.assertRaises(HttpResponseError) as cm:
            # create account with encryption value
            self.cmd("az netappfiles account change-key-vault -g {rg} -a {acc2_name} --key-vault-uri {keyVaultUri} --key-name {keyName} --keyvault-resource-id {keyVaultResourceId} --key-vault-private-endpoints [0].private-endpoint-id={privateEndpointId} [0].virtual-network-id={privateEndpointId}  --yes")
        self.assertIn('AzureKeyVaultEncryptionNotConfigured', str(
            cm.exception))


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from azure.cli.testsdk import (
    ScenarioTest, ResourceGroupPreparer, LiveScenarioTest, api_version_constraint,
    StorageAccountPreparer, JMESPathCheck, StringContainCheck, VirtualNetworkPreparer, KeyVaultPreparer)


class TestIdentity(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_identity_mgmt_')
    def test_identity_management(self, resource_group):
        self.kwargs.update({
            'identity': 'myidentity',
            'resource_restriction_compute': '\{"providers":\["Microsoft.Compute"\]\}',
            'resource_restriction_empty': '\{"providers":\[\]\}'
        })

        operations = self.cmd('identity list-operations').get_output_in_json()
        self.assertGreaterEqual(len(operations), 1)

        self.cmd('identity create -n {identity} -g {rg}', checks=[
            self.check('name', '{identity}'),
            self.check('resourceGroup', '{rg}'),
            self.check('isolationScope', 'None'),
            self.check('assignmentRestrictions.providers', [])
        ])

        self.cmd('identity list-resources -g {rg} -n {identity}')

        self.cmd('identity list -g {rg}', checks=self.check('length(@)', 1))
        self.cmd('identity delete -n {identity} -g {rg}')

        self.cmd('identity create -n {identity} -g {rg} --isolation-scope Regional', checks=[
            self.check('name', '{identity}'),
            self.check('resourceGroup', '{rg}'),
            self.check('isolationScope', 'Regional')
        ])

        self.cmd('identity update -n {identity} -g {rg} --isolation-scope None', checks=[
            self.check('name', '{identity}'),
            self.check('resourceGroup', '{rg}'),
            self.check('isolationScope', 'None')
        ])

        self.cmd('identity delete -n {identity} -g {rg}')

        self.cmd('identity create -n {identity} -g {rg} --resource-restriction {resource_restriction_compute}', checks=[
            self.check('name', '{identity}'),
            self.check('resourceGroup', '{rg}'),
            self.check('assignmentRestrictions.providers[0]', 'Microsoft.Compute')
        ])

        self.cmd('identity update -n {identity} -g {rg} --resource-restriction {resource_restriction_empty}', checks=[
            self.check('name', '{identity}'),
            self.check('resourceGroup', '{rg}'),
            self.check('assignmentRestrictions.providers', [])
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_federated_identity_credential_', location='centraluseuap')
    def test_federated_identity_credential(self, resource_group):
        self.kwargs.update({
            'identity': 'ide',
            'fic1': 'fic1',
            'fic2': 'fic2',
            'fic3': 'fic3',
            'fic4': 'fic4',
            'subject1': 'system:serviceaccount:ns:svcaccount1',
            'subject2': 'system:serviceaccount:ns:svcaccount2',
            'subject3': 'system:serviceaccount:ns:svcaccount3',
            'subject4': 'system:serviceaccount:ns:svcaccount4',
            'issuer': 'https://login.microsoftonline.com/72f988bf-86f1-41af-91ab-2d7cd011db47/v2.0',
            'audience': 'api://AzureADTokenExchange',
            'cme_version': '1',
            'cme_value': "claims['sub'] eq 'foo'",
        })

        self.cmd('identity create -n {identity} -g {rg}')

        # create a federated identity credential using subject
        self.cmd('identity federated-credential create --name {fic1} --identity-name {identity} --resource-group {rg} '
                 '--subject {subject1} --issuer {issuer} --audiences {audience}',
                 checks=[
                     self.check('length(audiences)', 1),
                     self.check('audiences[0]', '{audience}'),
                     self.check('issuer', '{issuer}'),
                     self.check('subject', '{subject1}')
                 ])

        # create another federated identity credential using subject
        self.cmd('identity federated-credential create --name {fic2} --identity-name {identity} --resource-group {rg} '
                 '--subject {subject2} --issuer {issuer} --audiences {audience}',
                 checks=[
                     self.check('length(audiences)', 1),
                     self.check('audiences[0]', '{audience}'),
                     self.check('issuer', '{issuer}'),
                     self.check('subject', '{subject2}')
                 ])

        # create a federated identity credential using claims matching expression
        self.cmd('identity federated-credential create --name {fic3} --identity-name {identity} --resource-group {rg} '
                 '--claims-matching-expression-version {cme_version} '
                 '--claims-matching-expression-value "{cme_value}" '
                 '--issuer {issuer} --audiences {audience}',
                 checks=[
                     self.check('length(audiences)', 1),
                     self.check('audiences[0]', '{audience}'),
                     self.check('issuer', '{issuer}'),
                     self.check('claimsMatchingExpression.languageVersion', 1),
                     self.check('claimsMatchingExpression.value', "{cme_value}")
                 ])

        # show the federated identity credential with subject
        self.cmd('identity federated-credential show --name {fic1} --identity-name {identity} --resource-group {rg}',
                 checks=[
                     self.check('length(audiences)', 1),
                     self.check('audiences[0]', '{audience}'),
                     self.check('issuer', '{issuer}'),
                     self.check('subject', '{subject1}')
                 ])

        # show the federated identity credential with claims matching expression
        self.cmd('identity federated-credential show --name {fic3} --identity-name {identity} --resource-group {rg}',
                 checks=[
                     self.check('length(audiences)', 1),
                     self.check('audiences[0]', '{audience}'),
                     self.check('issuer', '{issuer}'),
                     self.check('claimsMatchingExpression.languageVersion', 1),
                     self.check('claimsMatchingExpression.value', "{cme_value}")
                 ])

        # list the federated identity credentials
        self.cmd('identity federated-credential list --identity-name {identity} --resource-group {rg}',
                 checks=[
                     self.check('type(@)', 'array'),
                     self.check('length(@)', 3),
                     self.check('length([0].audiences)', '1'),
                     self.check('[0].audiences[0]', '{audience}'),
                     self.check('[0].issuer', '{issuer}'),
                     self.check('[0].subject', '{subject1}'),
                     self.check('length([1].audiences)', '1'),
                     self.check('[1].audiences[0]', '{audience}'),
                     self.check('[1].issuer', '{issuer}'),
                     self.check('[1].subject', '{subject2}'),
                     self.check('length([2].audiences)', '1'),
                     self.check('[2].audiences[0]', '{audience}'),
                     self.check('[2].issuer', '{issuer}'),
                     self.check('[2].claimsMatchingExpression.languageVersion', 1),
                     self.check('[2].claimsMatchingExpression.value', "{cme_value}")
                 ])

        # update a federated identity credential with subject to a different subject
        self.kwargs['new_subject'] = 'system:serviceaccount:ns:newaccount'
        self.cmd('identity federated-credential update --name {fic1} --identity-name {identity} --resource-group {rg} '
                 '--subject {new_subject} --issuer {issuer} --audiences {audience}',
                 checks=[
                     self.check('name', '{fic1}'),
                     self.check('subject', '{new_subject}')
                 ])

        # update a federated identity credential with claims matching expression to a different expression
        self.kwargs['new_cme_value'] = "claims['sub'] eq 'updatedFoo'"
        self.cmd('identity federated-credential update --name {fic3} --identity-name {identity} --resource-group {rg} '
                 '--claims-matching-expression-version {cme_version} '
                 '--claims-matching-expression-value "{new_cme_value}" '
                 '--issuer {issuer} --audiences {audience}',
                 checks=[
                     self.check('name', '{fic3}'),
                     self.check('claimsMatchingExpression.languageVersion', 1),
                     self.check('claimsMatchingExpression.value', "{new_cme_value}")
                 ])

        # delete first federated identity credential
        self.cmd('identity federated-credential delete --name {fic1}'
                 ' --identity-name {identity} --resource-group {rg} --yes')
                 
        # verify remaining credentials after first deletion
        self.cmd('identity federated-credential list --identity-name {identity} --resource-group {rg}',
                 checks=[
                     self.check('type(@)', 'array'),
                     self.check('length(@)', 2),
                     self.check('[0].name', '{fic2}'),
                     self.check('[0].subject', '{subject2}'),
                     self.check('[1].name', '{fic3}'),
                     self.check('[1].claimsMatchingExpression.value', "{new_cme_value}")
                 ])

        # test default audiences value
        self.cmd('identity federated-credential create --name {fic4} --identity-name {identity} --resource-group {rg} '
                 '--subject {subject4} --issuer {issuer}',
                 checks=[
                     self.check('length(audiences)', 1),
                     self.check('audiences[0]', 'api://AzureADTokenExchange'),
                     self.check('issuer', '{issuer}'),
                     self.check('subject', '{subject4}')
                 ])
        
        # delete remaining federated identity credentials
        self.cmd('identity federated-credential delete --name {fic2}'
                 ' --identity-name {identity} --resource-group {rg} --yes')
        self.cmd('identity federated-credential delete --name {fic3}'
                 ' --identity-name {identity} --resource-group {rg} --yes')
        self.cmd('identity federated-credential delete --name {fic4}'
                 ' --identity-name {identity} --resource-group {rg} --yes')
        
        self.cmd('identity federated-credential list --identity-name {identity} --resource-group {rg}',
                 checks=[
                     self.check('type(@)', 'array'),
                     self.check('length(@)', 0)
                 ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=too-many-lines
# pylint: disable=too-many-statements

from ..action import AddSoldTo


def load_arguments(self, _):

    with self.argument_context("billing instruction") as c:
        c.argument(
            "billing_account_name",     # override the parameter name comes from getter method
            options_list=["--account-name"],
            type=str,
            help="The ID that uniquely identifies a billing account.",
        )
        c.argument(
            "billing_profile_name",     # override the parameter name comes from getter method
            options_list=["--profile-name"],
            type=str,
            help="The ID that uniquely identifies a billing profile.",
        )
        c.argument(
            "instruction_name", options_list=["--name", "-n"], type=str, help="Instruction Name."
        )
        c.argument(
            "amount",
            type=float,
            help="The amount budgeted for this billing instruction.",
        )
        c.argument(
            "start_date", help="The date this billing instruction goes into effect."
        )
        c.argument(
            "end_date", help="The date this billing instruction is no longer in effect."
        )
        c.argument(
            "creation_date", help="The date this billing instruction was created."
        )

    with self.argument_context("billing invoice") as c:
        c.argument(
            "account_name", help="The ID that uniquely identifies a billing account"
        )
        c.argument("invoice_name", help="The ID that uniquely identifies an invoice")
        c.argument(
            "download_token",
            help="The download token with document source and document ID",
        )
        c.argument(
            "download_urls", nargs='+', help="Space-separated list of download urls for individual"
        )

    with self.argument_context("billing invoice show") as c:
        c.argument(
            "account_name",
            type=str,
            help="The ID that uniquely identifies a billing account.",
        )
        c.argument(
            "invoice_name",
            options_list=["--name", "-n", "--invoice-name"],
            type=str,
            help="The ID that uniquely identifies an invoice.",
        )
        c.argument(
            "by_subscription",
            action="store_true",
            help="When provided, it must work with --invoice-name to get an invoice by subscription ID and invoice ID",
        )

    with self.argument_context('billing invoice list') as c:
        c.argument('period_start_date', help='The start date to fetch the invoices. The date should be '
                   'specified in YYYY-MM-DD format.')
        c.argument('period_end_date', help='The end date to fetch the invoices. The date should be specified '
                   'in YYYY-MM-DD format.')

    with self.argument_context("billing policy") as c:
        c.argument(
            "account_name", help="The ID that uniquely identifies a billing account"
        )
        c.argument(
            "profile_name",
            type=str,
            help="The ID that uniquely identifies a billing profile.",
        )
        c.argument("customer_name", help="The ID that uniquely identifies a customer")

    with self.argument_context("billing profile create") as c:
        c.argument("bill_to", action=AddSoldTo, nargs="*", help="Billing address.")

    with self.argument_context("billing profile update") as c:
        c.argument("bill_to", action=AddSoldTo, nargs="*", help="Billing address.")

    with self.argument_context("billing role-definition") as c:
        c.argument(
            "name",
            options_list=["--name", "-n"],
            type=str,
            help="The name that uniquely identifies a role definition.",
        )
        c.argument(
            "account_name",
            type=str,
            help="The ID that uniquely identifies a billing account.",
        )
        c.argument(
            "profile_name",
            type=str,
            help="The ID that uniquely identifies a billing profile.",
        )
        c.argument(
            "invoice_section_name",
            type=str,
            help="The ID that uniquely identifies an invoice section.",
        )

    with self.argument_context("billing role-assignment") as c:
        c.argument(
            "account_name",
            type=str,
            help="The ID that uniquely identifies a billing account.",
        )
        c.argument(
            "name",
            options_list=["--name", "-n"],
            type=str,
            help="The ID that uniquely identifies a role assignment.",
        )
        c.argument(
            "profile_name",
            type=str,
            help="The ID that uniquely identifies a billing profile.",
        )
        c.argument(
            "invoice_section_name",
            type=str,
            help="The ID that uniquely identifies an invoice section.",
        )



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.command_modules.acs._client_factory import get_graph_client
from azure.cli.command_modules.acs._helpers import _get_test_sp_object_id
from azure.cli.core.azclierror import AzCLIError
from knack.log import get_logger

logger = get_logger(__name__)


def resolve_object_id(cli_ctx, assignee):
    if assignee is None:
        raise AzCLIError('Inputted parameter "assignee" is None.')
    # the assignee is the test sp
    object_id = _get_test_sp_object_id(assignee)
    if object_id:
        return object_id
    client = get_graph_client(cli_ctx)
    result = None
    # looks like a user principal name, find by upn (e.g., xxx@xxx.xxx)
    if assignee.find("@") >= 0:
        result = list(client.user_list(filter="userPrincipalName eq '{}'".format(assignee)))
    # find by spn (e.g., appId like xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx)
    if not result:
        result = list(client.service_principal_list(filter="servicePrincipalNames/any(c:c eq '{}')".format(assignee)))
    # find by display name (e.g., my-aks-sp)
    if not result:
        result = list(client.service_principal_list(filter="displayName eq '{}'".format(assignee)))
    # find by object id (e.g., (object) id like xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx)
    if not result:
        body = {"ids": [assignee], "types": ["user", "group", "servicePrincipal", "directoryObjectPartnerReference"]}
        result = list(client.directory_object_get_by_ids(body))

    # 2+ matches should never happen, so we only check 'no match' here
    if not result:
        raise AzCLIError("No matches in graph database for '{}'".format(assignee))
    return result[0]["id"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest
import tempfile
import shutil
import stat
import datetime
import time

import portalocker

from azure.cli.telemetry.components.telemetry_note import TelemetryNote


class TestTelemetryNote(unittest.TestCase):
    SAMPLE_TIME_1 = datetime.datetime(year=1999, month=12, day=31, hour=8, minute=12, second=13)
    SAMPLE_TIME_2 = datetime.datetime(year=2001, month=8, day=7, hour=21, minute=2, second=47)

    def setUp(self):
        self.workDir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.workDir, ignore_errors=True)

    def test_create_telemetry_note_file_from_scratch(self):
        with TelemetryNote(self.workDir) as note:
            self.assert_modify_time(note.path, datetime.timedelta(seconds=1))
            self.assertTrue(os.path.exists(note.path))
            self.assertEqual(datetime.datetime.min, note.get_last_sent())

            with self.assertRaises(portalocker.AlreadyLocked):
                with TelemetryNote(self.workDir):
                    self.assertFalse(True)

            self.set_modify_time(note.path)

            note.touch()
            self.assert_modify_time(note.path, datetime.timedelta(seconds=1))

            note.update_telemetry_note(self.SAMPLE_TIME_1)

            # newly create file is not readable
            self.assertEqual(datetime.datetime.min, note.get_last_sent())

        with TelemetryNote(self.workDir) as note:
            self.assertEqual(self.SAMPLE_TIME_1, note.get_last_sent())

            with self.assertRaises(portalocker.AlreadyLocked):
                with TelemetryNote(self.workDir):
                    self.assertFalse(True)

    def test_open_telemetry_note_file(self):
        with open(TelemetryNote.get_file_path(self.workDir), mode='w') as fq:
            fq.write(self.SAMPLE_TIME_1.strftime('%Y-%m-%dT%H:%M:%S'))

        with TelemetryNote(self.workDir) as note:
            self.assertEqual(self.SAMPLE_TIME_1, note.get_last_sent())

            with self.assertRaises(portalocker.AlreadyLocked):
                with TelemetryNote(self.workDir):
                    self.assertFalse(True)

            note.update_telemetry_note(self.SAMPLE_TIME_2)

        with TelemetryNote(self.workDir) as note:
            self.assertEqual(self.SAMPLE_TIME_2, note.get_last_sent())

    def assert_modify_time(self, file_path, gap):
        modify_time = datetime.datetime.fromtimestamp(os.stat(file_path).st_mtime)
        self.assertTrue(datetime.datetime.now() - modify_time < gap)

    def set_modify_time(self, file_path, new_time=None):
        if not new_time:
            new_time = (datetime.datetime.now() - datetime.timedelta(minutes=5))

        expected_timestamp = time.mktime(new_time.timetuple()) + new_time.microsecond / 1E6
        os.utime(file_path, (expected_timestamp, expected_timestamp))

        modify_time = datetime.datetime.fromtimestamp(os.stat(file_path).st_mtime)
        actual_timestamp = time.mktime(modify_time.timetuple()) + modify_time.microsecond / 1E6
        self.assertAlmostEqual(expected_timestamp, actual_timestamp, delta=1E-6)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# from azure.core.exceptions import ResourceExistsError, ResourceNotFoundError
from azure.cli.core.azclierror import UserFault

from ._base import has_value
from .exceptions import AAZUnknownFieldError


class AAZSelectors:

    def __init__(self):
        self._selectors = {}

    def __getattr__(self, key):
        if key not in self._selectors:
            raise AAZUnknownFieldError(self, key)
        return self._selectors[key]

    def __setattr__(self, key, value):
        if key.startswith('_'):
            self.__dict__[key] = value
        else:
            self._selectors[key] = value


class AAZSelector:  # pylint: disable=too-few-public-methods

    def __init__(self, ctx, name):
        self.ctx = ctx
        setattr(ctx.selectors, name, self)


class AAZJsonSelector(AAZSelector):

    def _get(self):
        raise NotImplementedError()

    def _set(self, value):
        raise NotImplementedError()

    def get(self):
        try:
            return self._get()
        except StopIteration:
            raise UserFault("ResourceNotFoundError")

    def set(self, value):
        try:
            self._set(value)
        except StopIteration:
            raise UserFault("ResourceNotFoundError")

    def required(self):
        value = self.get()
        if not has_value(value):
            raise UserFault("ResourceNotFoundError")
        return value



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
from unittest.mock import Mock, patch

from azure.cli.command_modules.acs.azuremonitormetrics.amw.defaults import (
    get_default_mac_name_and_region, 
    get_default_mac_region
)

class TestAMWDefaults(unittest.TestCase):
    @patch('azure.cli.command_modules.acs.azuremonitormetrics.amw.defaults.get_supported_rp_locations')
    def test_get_default_mac_region(self, mock_get_supported_rp_locations):
        mock_get_supported_rp_locations.return_value = ["eastus", "westus"]

        self.assertEqual(get_default_mac_region(
            cmd=None, cluster_region='eastus', subscription='test_subscription'
        ), 'eastus')
        self.assertEqual(get_default_mac_region(
            cmd=None, cluster_region='centraluseuap', subscription='test_subscription'
        ), 'eastus2euap')
        self.assertEqual(get_default_mac_region(
            cmd=None, cluster_region='nonexistent', subscription='test_subscription'
        ), 'eastus')

    def test_get_default_mac_name_and_region(self):
        with patch('azure.cli.command_modules.acs.azuremonitormetrics.amw.defaults.get_default_mac_region') as mock_get_default_mac_region:
            mock_get_default_mac_region.return_value = 'eastus'

            default_mac_name, default_mac_region = get_default_mac_name_and_region(
                cmd=None, cluster_region='eastus', subscription='test_subscription'
            )

            self.assertEqual(default_mac_name, 'DefaultAzureMonitorWorkspace-eastus')
            self.assertEqual(default_mac_region, 'eastus')

if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest
from unittest import mock

from azure.cli.core.azclierror import CLIError
from azure.cli.command_modules.acr.custom import _validate_command_path


class TestValidateCommandPath(unittest.TestCase):
    """Tests for _validate_command_path CWD validation logic."""

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_binary_in_cwd_is_blocked(self, mock_getcwd, mock_which):
        """A docker binary located directly in CWD should be rejected."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/home/user/repo/docker'

        with self.assertRaises(CLIError) as ctx:
            _validate_command_path('docker', is_diagnostics_context=False)
        self.assertIn('Refusing to use', str(ctx.exception))

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_binary_in_system_path_is_allowed(self, mock_getcwd, mock_which):
        """A docker binary in a system directory should be allowed."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/usr/bin/docker'

        result = _validate_command_path('docker', is_diagnostics_context=False)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_binary_in_subdirectory_of_cwd_is_allowed(self, mock_getcwd, mock_which):
        """A docker binary in a subdirectory of CWD should be allowed (only direct CWD is blocked)."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/home/user/repo/tools/docker'

        result = _validate_command_path('docker', is_diagnostics_context=False)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_binary_at_root_cwd_is_allowed(self, mock_getcwd, mock_which):
        """When CWD is filesystem root, system docker should not be blocked."""
        mock_getcwd.return_value = '/'
        mock_which.return_value = '/usr/bin/docker'

        result = _validate_command_path('docker', is_diagnostics_context=False)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    def test_binary_not_found_is_allowed(self, mock_which):
        """When docker is not found at all, validation should pass (let downstream handle it)."""
        mock_which.return_value = None

        result = _validate_command_path('docker', is_diagnostics_context=False)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_diagnostics_mode_returns_true_on_unsafe(self, mock_getcwd, mock_which):
        """In diagnostics mode, should return True (unsafe) instead of raising."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/home/user/repo/docker'

        result = _validate_command_path('docker', is_diagnostics_context=True)
        self.assertTrue(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_diagnostics_mode_returns_false_on_safe(self, mock_getcwd, mock_which):
        """In diagnostics mode, should return False (safe) for system-path docker."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/usr/bin/docker'

        result = _validate_command_path('docker', is_diagnostics_context=True)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_windows_cwd_binary_is_blocked(self, mock_getcwd, mock_which):
        """A docker binary (with platform extension) in CWD should be blocked."""
        if os.name == 'nt':
            mock_getcwd.return_value = 'C:\\Users\\dev\\repo'
            mock_which.return_value = 'C:\\Users\\dev\\repo\\docker.exe'
        else:
            mock_getcwd.return_value = '/home/dev/repo'
            mock_which.return_value = '/home/dev/repo/docker'

        with self.assertRaises(CLIError):
            _validate_command_path('docker', is_diagnostics_context=False)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_windows_system_docker_is_allowed(self, mock_getcwd, mock_which):
        """Docker in a system directory (not CWD) should be allowed."""
        if os.name == 'nt':
            mock_getcwd.return_value = 'C:\\Users\\dev\\repo'
            mock_which.return_value = 'C:\\Program Files\\Docker\\docker.exe'
        else:
            mock_getcwd.return_value = '/home/dev/repo'
            mock_which.return_value = '/usr/local/bin/docker'

        result = _validate_command_path('docker', is_diagnostics_context=False)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_windows_root_cwd_does_not_block_system_docker(self, mock_getcwd, mock_which):
        """When CWD is the filesystem root, system docker should not be blocked."""
        if os.name == 'nt':
            mock_getcwd.return_value = 'C:\\'
            mock_which.return_value = 'C:\\Program Files\\Docker\\docker.exe'
        else:
            mock_getcwd.return_value = '/'
            mock_which.return_value = '/usr/bin/docker'

        result = _validate_command_path('docker', is_diagnostics_context=False)
        self.assertFalse(result)

    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_error_message_includes_override_instruction(self, mock_getcwd, mock_which):
        """Error message should tell user how to override with DOCKER_COMMAND."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/home/user/repo/docker'

        with self.assertRaises(CLIError) as ctx:
            _validate_command_path('docker', is_diagnostics_context=False)
        self.assertIn('DOCKER_COMMAND', str(ctx.exception))
        self.assertIn('absolute path', str(ctx.exception))

    @mock.patch('azure.cli.command_modules.acr.custom.os.path.realpath')
    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_symlink_in_cwd_pointing_to_system_binary_is_blocked(self, mock_getcwd, mock_which, mock_realpath):
        """A symlink in CWD pointing to /usr/bin/docker should still be blocked (uses abspath, not realpath)."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/home/user/repo/docker'
        mock_realpath.return_value = '/usr/bin/docker'

        with self.assertRaises(CLIError):
            _validate_command_path('docker', is_diagnostics_context=False)

    @mock.patch('azure.cli.command_modules.acr.custom.os.path.realpath')
    @mock.patch('azure.cli.command_modules.acr.custom.os.path.abspath')
    @mock.patch('azure.cli.command_modules.acr.custom.shutil.which')
    @mock.patch('azure.cli.command_modules.acr.custom.os.getcwd')
    def test_system_binary_symlink_resolving_to_cwd_is_blocked(self, mock_getcwd, mock_which, mock_abspath,
                                                               mock_realpath):
        """A binary outside CWD whose realpath resolves into the CWD should be blocked."""
        mock_getcwd.return_value = '/home/user/repo'
        mock_which.return_value = '/usr/local/bin/docker'
        mock_abspath.side_effect = lambda p: p if p != '.' else '/home/user/repo'
        mock_realpath.return_value = '/home/user/repo/docker'

        with self.assertRaises(CLIError):
            _validate_command_path('docker', is_diagnostics_context=False)
